@@ -3,12 +3,13 @@ use warnings;
 use strict;
 use Carp;
 use utf8;
-our $VERSION = '0.00_02';
+our $VERSION = '0.00_03';
 
 use IO::Socket;
 use JSON::Create ':all';
 use JSON::Parse ':all';
 use Unicode::UTF8 'decode_utf8';
+use boolean;
 
 $SIG{PIPE} = sub {
     croak "Aborting on SIGPIPE";
@@ -17,7 +18,9 @@ $SIG{PIPE} = sub {
 sub set_opt
 {
     my ($gs, $o, $nm) = @_;
-    if ($o->{$nm}) {
+    # Use exists here so that, e.g. verbose => $verbose, $verbose =
+    # undef works OK.
+    if (exists $o->{$nm}) {
 	$gs->{$nm} = $o->{$nm};
 	delete $o->{$nm};
     }
@@ -38,7 +41,12 @@ sub new
     if (! $gs->{port}) {
 	carp "No port specified";
     }
-    $gs->{jc} = JSON::Create->new (indent => 1, sort => 1, downgrade_utf8 => 1);
+    $gs->{jc} = JSON::Create->new (
+	indent => 1,
+	sort => 1,
+	downgrade_utf8 => 1,
+    );
+    $gs->{jc}->bool ('boolean');
     $gs->{jp} = JSON::Parse->new ();
     return bless $gs;
 }
@@ -57,6 +65,9 @@ sub serve
     if (! $gs->{server}) {
 	carp "Can't open socket: $@";
     }
+    if ($gs->{verbose}) {
+	vmsg ("Serving on $gs->{port}");
+    }
     while (1) {
 	my $got = '';
 	my ($ok) = eval {
@@ -94,6 +105,20 @@ sub serve
 	    vmsg ("Validated as JSON");
 	}
 	my $input = $gs->{jp}->parse ($got);
+	my $control = $input->{'JSON::Server::control'};
+	if (defined $control) {
+	    if ($control eq 'stop') {
+		if ($gs->{verbose}) {
+		    vmsg ("Received control message to stop");
+		}
+		$gs->reply ({'JSON::Server::response' => 'stopping'});
+		if ($gs->{verbose}) {
+		    vmsg ("Responded to control message to stop");
+		}
+		return;
+	    }
+	    warn "Unknown control command '$control'";
+	}
 	$gs->respond ($input);
     }
 }
@@ -140,7 +165,7 @@ sub echo
 sub vmsg
 {
     my ($msg) = @_;
-    print "$msg.\n";
+    print __PACKAGE__ . ": $msg.\n";
 }
 
 
